#!/usr/bin/python
# -*- coding: utf-8 -*-

from ... import ftdi

from . import constant
from .common import common

#########################################
CONFIG_LIST = [
	#common
	['PDSW', constant.CHNUM_CMN],
	#CH
	['ADG', constant.CHNUM_CHN], ['THR', constant.CHNUM_CHN],
	['LLD', constant.CHNUM_CHN], ['ULD', constant.CHNUM_CHN],
	['OFS', constant.CHNUM_CHN]
]

class config(object):

	#########################################################
	def __init__(self, usbmca):
		self.mConfigList = CONFIG_LIST
		self.__initializing()
		self.mUSBMCA = usbmca

	#########################################################
	def __initializing(self):
		self.mUSBMCA = None
		self.mParameter = {}
		self.mLastException = None
		self.clearConfig()

	#########################################################
	def clearConfig(self):

		self.mParameter = {}
		zerovalue = [ 0 for n in range(constant.CHNUM_CNT) ]

		for name, chn in self.mConfigList:
			if chn == constant.CHNUM_CHN:
				self.mParameter[name,] = zerovalue
			else:
				self.mParameter[name,] = 0

		return

	#########################################################
	def __getUseDestination(self, dest):
		_dest = dest if dest is not None else self.mUSBMCA
		return _dest

	#########################################################
	def setDefaultDest(self, usbmca):
		self.mUSBMCA = usbmca

	#########################################################
	def startAcquisition(self, dest = None):
		return self.writeParameter('AQSW', 1, constant.CHNUM_CMN, dest)

	#########################################################
	def stopAcquisition(self, dest = None):
		return self.writeParameter('AQEW', 1, constant.CHNUM_CMN, dest)

	#########################################################
	def clearAcquisitionData(self, dest = None):
		return self.writeParameter('CLRW', 1, constant.CHNUM_CMN, dest)

	#########################################################
	def setAcquisitionMode(self, mode, dest = None):
		return self.writeParameter('MODW', mode, constant.CHNUM_CMN, dest)

	#########################################################
	def setAcquisitionTimeMode(self, mode = constant.PARAM_MMD_REALTIME, dest = None):
		return self.writeParameter('MMDW', mode, constant.CHNUM_CMN, dest)

	#########################################################
	def setAcquisitionTime(self, seconds, dest = None):
		
		result = False
		digit = common.convertTimeSecondToDigits(seconds)
		digit0 = (digit >> 32) & 0x0fff
		digit1 = (digit >>  0) & 0x0ffffffff

		ret1st = self.writeParameter('MT0W', digit0, constant.CHNUM_CMN, dest)		#MSB

		if ret1st == True:
			result = self.writeParameter('MT1W', digit1, constant.CHNUM_CMN, dest)	#LSB

		return result

	#########################################################
	def writeParameter(self, command, value, chn, dest = None):
		_dest = self.__getUseDestination(dest)
		result = _dest.RequestCommand(command, value, chn)

		if result == True:
			try:
				dummy = self.mParameter[command,]
				if chn == constant.CHNUM_CMN:
					self.mParameter[command,] = value
				else:
					self.mParameter[command,][chn-1] = value
			except Exception as ex:
				self.mLastException = ex

		return result
	
	#########################################################
	def writeParameterAll(self, dest = None):
		
		result = True

		for name, in self.mParameter:
			if isinstance(self.mParameter[name,], list):
				value = self.mParameter[name,]		#CH unit
				chn = [ constant.CHNUM_1ST + n for n in range(len(value))]
			else:
				value = [ self.mParameter[name,] ]	#common
				chn = [ constant.CHNUM_CMN ]

			for i in range(len(value)):
				result = self.writeParameter(name, value[i], chn[i], dest)
				
				if result == False:
					return result

		return result

